##
##   Copyright (C) 2006 Mario Theodoridis, mario@schmut.com
##
##   This "Original Work" is free; you can modify it under the terms of the
##   AFL Academic Free License. This "Original Work" is distributed in the
##   hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
##   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##   See the AFL Academic Free License for more details. You should find a
##   copy of the AFL Academic Free License in the highest level directory of
##   this distribution; if not, you may obtain one at the Open Source
##   Initiative, http://www.opensource.org.
##

from xmltreewalker.operator import Operator
from sys import stdout
import re

class DocbookTocOperator(Operator):
    """Docbook Table of Contents Parser
    
    This class will parse out a <dl><dt><a href='url'>name</a></dt><dd><dl><dt>..
    type definition list and return a list of TocItems representing the TOC
    data parsed.

    All I/O is in unicode.
    """
    __name__ = 'DocbookTocOperator'
    DL = 'dl'
    A = 'a'
    HREF = 'href'

    def __init__(self, isActive = True, filterList=[], bubbles=True):
        '''Contstructor to specify on/off state and filters
        
        The initial on/off state
        The filter list specifies when to toggle the on/off state
        
        '''
        super(DocbookTocOperator, self).__init__(isActive, filterList, bubbles)
        self.rootItem = TocItem()
        self.rootItem.name = "Root"
        self.itemStack = [self.rootItem] 
        self.isInAnchor = False


    def startHandler(self, name, attrs):
        if name == self.DL:
            item = self.itemStack[-1]
            item.initList()
            self.itemStack.append(None)
        elif name == self.A:
            map = self.__getMap(attrs)
            url = map[self.HREF] or None
            item = TocItem(url)
            parent = self.itemStack[-2]
            parent.append(item)
            self.itemStack[-1] = item
            self.isInAnchor = True
        return self.bubbles


    def dataHandler(self, data):
        if self.isInAnchor:
            self.itemStack[-1].name += data
        return self.bubbles


    def endHandler(self, name):
        if name == self.DL:
            self.itemStack.pop()
        elif name == self.A:
            self.isInAnchor = False
        return self.bubbles


    def __getMap(self, list):
        '''turn the list of key value pairs into a map'''
        myMap = {}
        for i in range(0,len(list),2):
            myMap[list[i]] = list[i+1]
        return myMap


class TocItem(object):
    '''A TOC Item represents a Table of Contents Item as parsed by 
    DocbookTocOperator.
    
    Each item has a url, name and an optional list of child items.
    '''
    def __init__(self, url = None):
        self.url = url
        self.name = ""
        self.children = None


    def append(self, item):
        if self.children is None:
            raise StateException("List not defined")
        self.children.append(item)


    def initList(self):
        if self.children is not None:
            raise StateException("List already defined")
        self.children = []


    def __repr__(self):
        return u"url: %s - name: %s -children: %s" % (self.url, self.name, 
                                                  self.children)


    def getName(self):
        return re.sub(r"[\r\n\s]+", " ", self.name)


    def getUrl():
        return self.url


    def prettyPrint(self, indent = ""):
        ret = indent + "URL: %s - Name: %s" % (self.url, self.getName())
        if self.children is None:
            return ret + '\n'
        ret += indent + " - Children:\n"
        for ti in self.children:
            ret += ti.prettyPrint(indent + "  ")
        return ret
        


class StateException(Exception):
    def __init__(self, msg=""):
        self.msg = msg
    def __repr__(self):
        return self.msg


if __name__ == '__main__':
    ti1 = TocItem('url1')
    ti1.name = 'name1'
    ti1.initList()
    ti2 = TocItem('url2')
    ti2.name = 'multiline\nname2'
    ti1.append(ti2)
    print ti1
    print ti1.prettyPrint()
    
    