##
##   Copyright (C) 2006 Mario Theodoridis, mario@schmut.com
##
##   This "Original Work" is free; you can modify it under the terms of the
##   AFL Academic Free License. This "Original Work" is distributed in the
##   hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
##   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##   See the AFL Academic Free License for more details. You should find a
##   copy of the AFL Academic Free License in the highest level directory of
##   this distribution; if not, you may obtain one at the Open Source
##   Initiative, http://www.opensource.org.
##

from xmltreewalker.tagrequirement import TagRequirement
from xmltreewalker.tag import Tag
from types import *


class Filter:


    def __init__(self, tagReqs):
        self.tagReqs = tagReqs


    def test(self, tagStack):
        reqLen = len(self.tagReqs)
        if len(tagStack) < reqLen: return False
        for i in range(reqLen):
            if not self.tagReqs[i].matches(tagStack[i]): return False
        return True

    @staticmethod
    def getInstance(reqList):
        '''Constructs a filter from supplied TagRequirements List
        
        Here's an example with everything specified
        >>> flt = Filter.getInstance([['t1', {'p1':'v1'}],['t2', {'p1':None}]])
        >>> flt
        [t1 {'p1': 'v1'}, t2 {'p1': None}]
        >>> 

        Here's one that leaves the tagname out by setting it to None
        >>> flt = Filter.getInstance([[None, {'p1':'v1'}]])
        >>> flt
        [None {'p1': 'v1'}]
        >>> 

        Here's one that leaves the tagname out altogether
        >>> flt = Filter.getInstance([[{'p1':'v1'}]])
        >>> flt
        [None {'p1': 'v1'}]
        >>> 

        Here's one that leaves the properties out by specifying an empty list
        >>> flt = Filter.getInstance([['tag1', {}]])
        >>> flt
        [tag1 {}]
        >>> 

        Here's one that leaves the properties out altogether
        >>> flt = Filter.getInstance([['tag1']])
        >>> flt
        [tag1 {}]
        >>> 

        Here's one that leaves everything out. Note a list is required
        without it the c'tor might as well be used
        >>> flt = Filter.getInstance([])
        >>> flt
        []
        >>> 

        Here's an example with everything specified
        >>> flt = Filter.getInstance([[],['t2', {'p1':None}]])
        >>> flt
        [None {}, t2 {'p1': None}]
        >>> 

        '''
        inList = []
        for bundle in reqList:
            name = None
            attrs = {}
            for item in bundle:
                if type(item) is StringType:
                    name = item
                elif type(item) is DictType:
                    attrs = item
            inList.append(TagRequirement(name, attrs))
        return Filter(inList)


    def __repr__(self):
        return repr(self.tagReqs)


if __name__ == '__main__':

    def test(flt, stk, res):
        print '%s  > %s filter [%s] matches stack [%s]' % (res, flt.test(stk), flt, stk)

    tag1 = Tag('test1')
    tag2 = Tag('test2', ['prop1' , 'val1'])
    tag3 = Tag('test3', ['prop1' , 'val1', 'prop2' , 'val2'])

    req1 = TagRequirement()
    req2 = TagRequirement('test1')
    req3 = TagRequirement(None, {'prop2':None})

    stk = [[tag1, tag2],
           [tag1, tag2, tag3],
           [tag1, tag3],
           [tag2, tag3],
       ]

    flt = [Filter([req1]),
           Filter([req2]),
           Filter([req2, req3]),
       ]

    res = [
        [True, True, True, True],
        [True, True, True, False],
        [False, False, True, False]]
    
    for f in range(len(flt)):
        for s in range(len(stk)):
            test(flt[f], stk[s], res[f][s])

    import doctest
    doctest.testmod()



