##
##   Copyright (C) 2006 Mario Theodoridis, mario@schmut.com
##
##   This "Original Work" is free; you can modify it under the terms of the
##   AFL Academic Free License. This "Original Work" is distributed in the
##   hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
##   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##   See the AFL Academic Free License for more details. You should find a
##   copy of the AFL Academic Free License in the highest level directory of
##   this distribution; if not, you may obtain one at the Open Source
##   Initiative, http://www.opensource.org.
##


from os.path import join, abspath, dirname, basename, exists
from StringIO import StringIO

from xmltreewalker.operator import Operator
from xmltreewalker.printoperator import PrintOperator
from xmltreewalker.collateral_printoperator import CollateralPrintOperator
from xmltreewalker.filter import Filter
from xmltreewalker.tagrequirement import TagRequirement
from xmltreewalker.walker import XMLTreeWalker
from xmltreewalker.docbook_toc_operator import DocbookTocOperator, TocItem

import unittest
from xmltreewalker.tests.utils import checkData, getUnicodeInput, normalize_html, \
     compareStrings

class XMLTreeWalkerTestCase(unittest.TestCase):
    
    def test_nullOperator(self):

        orig = getUnicodeInput(inpath('test1.html'))
        if orig is None:
            self.assert_(0)
            
        ops = [Operator(True, [])]
        wlk = XMLTreeWalker(ops)
        wlk.parse(orig)
    

    def test_passthrough(self):

        orig = getUnicodeInput(inpath('test2.xml'))
        if orig is None:
            self.assert_(0)
            
        data = StringIO()
        ops = [PrintOperator(True, [], data)]
        wlk = XMLTreeWalker(ops)
        wlk.parse(orig)
    
        got = data.getvalue()
        data.close()
        expected = normalize_html(orig)
        got = normalize_html(got)
        
        if got != expected:
            self.assert_(0, compareStrings(expected, got, 20,20))


    def test_splitting(self):

        orig = getUnicodeInput(inpath('test1.html'))
        if orig is None:
            self.assert_(0)
            
        flt = Filter.getInstance([['html'], ['body'],
                                   ['div', {'class': 'article'}],
                                   ['div', {'class':'toc'}],
                                   ['dl'],
                               ])

        flt2 = Filter.getInstance([['html'], ['body'],
                                   ['div', {'class': 'article'}],
                                   ['div', {'class':'toc'}],
                               ])

        data = StringIO()
        data2 = StringIO()
        ops = [PrintOperator(False, [flt], data),
               CollateralPrintOperator(join(PREFIX, 'input'), True, [flt2], data2)]
        wlk = XMLTreeWalker(ops)
        wlk.parse(orig)
    
        got = data.getvalue()
        data.close()
        self.checkData('test1_toc.html', got)
        got2 = data2.getvalue()
        data2.close()
        self.checkData('test1_notoc_ximg.html', got2)

    def test_tocOperator(self):

        orig = getUnicodeInput(inpath('test1.html'))
        if orig is None:
            self.assert_(0)
            
        zapflt = Filter.getInstance([['html'], ['body'], ['div'], ['div'],
                                     ['div'], ['div'],
                                     ['h1', {'class': 'title'}]])

        flt = Filter.getInstance([['html'], ['body'],
                                   ['div', {'class': 'article'}],
                                   ['div', {'class':'toc'}],
                                   ['dl'],
                               ])

        flt2 = Filter.getInstance([['html'], ['body'],
                                   ['div', {'class': 'article'}],
                                   ['div', {'class':'toc'}],
                               ])

        data = StringIO()
        dto = DocbookTocOperator(False, [flt])
        ops = [Operator(False, [zapflt], bubbles=False), dto, 
               PrintOperator(True, [flt2], data)]
        wlk = XMLTreeWalker(ops)
        wlk.parse(orig)
    
        got = data.getvalue()
        data.close()
        self.checkData('test1_notoc.html', got)
        self.checkData('test1_parsed_toc.txt', dto.rootItem.prettyPrint())
        
    def test_multiTag(self):

        orig = getUnicodeInput(inpath('test2.xml'))
        if orig is None:
            self.assert_(0)
            
        flt = Filter.getInstance([['article'], ['sect1'] ])

        data = StringIO()
        ops = [PrintOperator(False, [flt], data)]
        wlk = XMLTreeWalker(ops)
        wlk.parse(orig)
    
        got = data.getvalue()
        data.close()
        self.checkData('test2_multiTag.xml', got)
        
    def test_stripTag(self):

        orig = getUnicodeInput(inpath('test2.xml'))
        if orig is None:
            self.assert_(0)
            
        flt = Filter.getInstance([['article'], ['articleinfo'], ['title'], ])

        data = StringIO()
        ops = [PrintOperator(True, [flt], data)]
        wlk = XMLTreeWalker(ops)
        wlk.parse(orig)
    
        got = data.getvalue()
        data.close()
        self.checkData('test2_stripTag.xml', got)
        
    def checkData(self, path, got):
        '''Redirecter to the function'''
        checkData(self, outpath(path), got)

PREFIX = abspath(dirname(__file__))

def inpath(file):
    return join(PREFIX, 'input', file)

def outpath(file):
    return join(PREFIX, 'output', file)

        

def test_suite():
    return unittest.makeSuite(XMLTreeWalkerTestCase)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")
