
import os
from os.path import join, abspath, dirname, basename, exists
import codecs
import re
import unittest


def checkData(testcase, outPath, got):
    '''Does the dirty work of checking output files against expected
    
    testcase - unittest instance
    outPath - filepath to file to compare data to or write data if not found.
    got - unicode data to be tested
    '''
    (res, expected) = getSavedData(outPath, got)
    if not res:
        testcase.assert_(0)
    expected = normalize_html(expected)
    got = normalize_html(got)

    if got != expected:
        testcase.assert_(0, compareStrings(expected, got, 50,50))


def compareStrings(expected, got, before=10, after=10):
    r'''Diffs a string and returns the part that varied surrounded by 'before'
    and 'after' number of characters.
    Helpful for finding differences in large strings.
    
    >>> print compareStrings("abcde", "abcde")
    <BLANKLINE>
    The strings match
    >>> print compareStrings("abcde", "abcdef")
    <BLANKLINE>
    The string matched but there was more data aferwards [f]
    >>> print compareStrings("abcdef", "abcde")
    <BLANKLINE>
    The string is too short. Matched up to offset [5] or line [1]
    [f]
    >>> print compareStrings("abcde", "abxde")
    <BLANKLINE>
    At offset [2] or line #1
    [abcde]
       ^
    [abxde]
    <BLANKLINE>
    >>>
    '''
    len1 = len(expected)
    len2 = len(got)
    if len1 < len2:
        last = len1
    else:
        last = len2
    line = 1
    # check the strings up to the length of the shorter one
    for i in range(last):
        if got[i] == '\n':
            line += 1
        if expected[i] != got[i]:
            start = i-before
            end = i+after
            point = before + 1
            if start < 0:
                point += start
                start = 0
            pointStr = '%d' % point
            formatStr = "\nAt offset [%d] or line #%d\n[%s]\n %"+ pointStr +"s\n[%s]\n"
            return formatStr % (i, line, expected[start:end], '^', got[start:end])

    # we're here because both strings have a common beginning
    if len1 == len2:
        return "\nThe strings match"

    if len1 < len2:
        # extra stuff at the end
        end = len1+after
        if end > len2:
            end = len2
        return "\nThe string matched but there was more data aferwards [%s]" %\
               got[len1:end]

    # didn't get all of it
    end = len2+after
    if end > len1:
        end = len1
    return "\nThe string is too short. Matched up to offset [%d] or line [%d]\n[%s]"\
           % (len2, line, expected[len2:end])


def getSavedData(outPath, data):
    '''Returns expected data from outPath and writes given data to outPath
    if that file doesn't exist.
    On success returns a tuple of (resultCode True, expectedData)
    Else (resultCode False, "")
    The data is un unicode
    '''
    try:
        output = codecs.open(outPath, 'rb', 'utf-8')
    except IOError:
        import sys
        print >>sys.stderr, 'No output file found.'
        print >>sys.stderr, 'File %s created, check it!' % outPath
        output = codecs.open(outPath, 'w', 'utf-8')
        output.write(data)
        output.close()
        return (False, "");
    else:
        ret = output.read()
        output.close()
        return (True, ret)


def getUnicodeInput(path):
    '''Returns contents of UTF-8 encoded file as unicode'''
    
    try:
        input = codecs.open(path, 'rb', 'utf-8')
    except IOError, e:
        import sys
        print >>sys.stderr, "Can't open %s for reading because\n%s." % (path, e)
        return None
    else:
        orig = input.read()
        input.close()
        return orig

def normalize_html(s):
    """
    
    >>> print normalize_html('''
    ... <tag  >   some data<br /><br/><
    ... tag
    ... >
    ... more          data</tag>
    ... ''')
     <tag > some data<br/><br/>< tag > more data</tag> 
    
    """
    s = re.sub(r"\s+", " ", s)
    s = re.sub(r"(?s)\s+<", " <", s)
    s = re.sub(r"(?s)>\s+", "> ", s)
    s = re.sub(r"\r", "", s)
    s = re.sub(r"\s+/>", "/>", s)
    return s


def _test():
    import doctest
    doctest.testmod()


if __name__ == '__main__':
    _test()
