##
##   Copyright (C) 2006 Mario Theodoridis, mario@schmut.com
##
##   This "Original Work" is free; you can modify it under the terms of the
##   AFL Academic Free License. This "Original Work" is distributed in the
##   hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
##   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##   See the AFL Academic Free License for more details. You should find a
##   copy of the AFL Academic Free License in the highest level directory of
##   this distribution; if not, you may obtain one at the Open Source
##   Initiative, http://www.opensource.org.
##


from xml.parsers.expat import ParserCreate 
from xmltreewalker.operator import Operator
from xmltreewalker.printoperator import PrintOperator
from xmltreewalker.collateral_printoperator import CollateralPrintOperator
from xmltreewalker.tag import Tag
from xmltreewalker.filter import Filter
from xmltreewalker.tagrequirement import TagRequirement

from StringIO import StringIO
import logging

class XMLTreeWalker:
    """Walks over XML Document Invoking the specified operators
    All I/O is performed in unicode.
    This means the parse function expects unicode and
    all operator handlers are guaranteed to receive unicde.
    
    """
    __name__ = 'XMLTreeWalker'
    UTF8 = 'utf-8'
    
    def __init__(self, operatorList):
        self.__opList = operatorList
        self.__p = ParserCreate(self.UTF8)
        self.__p.SetParamEntityParsing(0)
        
        self.__p.StartElementHandler = self.startHandler
        self.__p.CharacterDataHandler = self.dataHandler
        self.__p.EndElementHandler = self.endHandler
        self.__p.ProcessingInstructionHandler = self.procHandler
        self.__p.DefaultHandler = self.defaultHandler
        self.__p.ordered_attributes = True
        self.__p.returns_unicode = True

    def defaultHandler(self, data):
        for operator in self.__opList:
            if operator.isActive:
                if not operator.defaultHandler(data):
                    break


    def procHandler(self, target, data):
        for operator in self.__opList:
            if operator.isActive:
                if not operator.procHandler(target, data):
                    break


    def parseFile(self, xmlFile):
        self.__tagStack = []
        self.__p.ParseFile(xmlFile)


    def parse(self, xmlData):
        '''Expects unicode data'''
        self.__tagStack = []
        self.__p.Parse(xmlData.encode(self.UTF8), 1)


    def startHandler(self, name, attrs):
        tag = Tag(name, attrs)
        self.__tagStack.append(tag)
        for operator in self.__opList:
            if operator.stateChanged(self.__tagStack):
                #logging.warn('appending %s to %s' %(operator.filters, tag))
                tag.append(operator)
    
        for operator in self.__opList:
            if operator.isActive:
                if not operator.startHandler(name, attrs):
                    break



    def dataHandler(self, data):
        for operator in self.__opList:
            if operator.isActive:
                if not operator.dataHandler(data):
                    break


    def endHandler(self, name):
        tag = self.__tagStack.pop()
        for operator in self.__opList:
            if operator.isActive:
                if not operator.endHandler(name):
                    break

        for operator in tag.activeList:
            #logging.warn('resetting %s in %s' %(operator.filters, tag))
            operator.resetState()


if __name__ == '__main__':
    from os.path import abspath, dirname
    import sys
    
    flt1 = Filter.getInstance([['parent'], ['child1', {'name' : None}], ['b']])
    flt2 = Filter.getInstance([['parent'], ['child2']])
    
    ops = [PrintOperator(isActive=False, filterList=[flt1, flt2])]
    ops = [PrintOperator(isActive=True)]
    wlk = XMLTreeWalker(ops)
    wlk.parse(r"""<?php 
    $prefix = $_SERVER['COMMON_PHP_HOME'].'php/';
    require_once($prefix.'common/docUtils.phi');
    echo '<?xml version="1.0" \\encoding="UTF-8"?'.">\\n";
    echo "\\t<child name="&lt;fred&quot;">&lt;key&gt;WC_FIELD_VALUE&lt;/key&gt;</child>\\n";
    ?>
    <!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN" "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd" [
      <!ENTITY widget "Widget Framework">
      ]>
    <parent prop="" id="top"><?python echo; ?><child1 name="paul"><p>Text <b>goes</b> here</p></child1>
    <child2 name="fred">More text</child2>
    <br/>
    </parent>""")
    print "\n\nmultitag test"

    flt = Filter.getInstance([['parent'], ['child']])
    
    ops = [PrintOperator(False, [flt])]
    wlk = XMLTreeWalker(ops)
    wlk.parse("""<!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN" "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd" [
      <!ENTITY widget "Widget Framework">
      ]>
    <parent prop="" id="top"><child name="paul">Text <b>goes</b> here</child>
    <child name="fred">More text</child>
    </parent>""")
    print
    
    print "\n\nstriptag test"

    flt = Filter.getInstance([['parent'], ['child']])
    
    ops = [PrintOperator(True, [flt])]
    wlk = XMLTreeWalker(ops)
    wlk.parse("""<!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN" "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd" [
      <!ENTITY widget "Widget Framework">
      ]>
    <parent prop="" id="top"><p>test</p>
    <child name="paul">Text <b>goes</b> here</child>
    </parent>""")
    print
    wlk = XMLTreeWalker(ops)
    wlk.parse("""<!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN" "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd" [
      <!ENTITY widget "Widget Framework">
      ]>
    <parent prop="" id="top"><child name="paul">Text <b>goes</b> here
    </child></parent>""")
    print
    wlk = XMLTreeWalker(ops)
    wlk.parse("""<!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN" "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd" [
      <!ENTITY widget "Widget Framework">
      ]>
    <parent prop="" id="top"><child name="paul">Text <b>goes</b> here</child>
    <p>test</p>
    </parent>""")
    print
    print "\n\nun escape test"

    flt = Filter.getInstance([['parent'], ['child']])

    ops = [CollateralPrintOperator(abspath(dirname(__file__)), False, [flt])]
    wlk = XMLTreeWalker(ops)
    wlk.parse("""<!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN" "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd" [
      <!ENTITY widget "Widget Framework">
      ]>
    <parent prop="" id="top"><child name="paul">Text <b>goes</b> here</child>
    <child name="&lt;fred&quot;">&lt;key&gt;WC_FIELD_VALUE&lt;/key&gt;</child>
    <child><img src='tests/input/test.gif'/></child>
    </parent>""")
    print

    print "\n\masking test"

    maskflt = Filter.getInstance([['parent'], ['child'], ['b']])
    flt = Filter.getInstance([['parent'], ['child']])

    ops = [Operator(False, [maskflt], bubbles=False), 
           PrintOperator(False, [flt])]
    wlk = XMLTreeWalker(ops)
    wlk.parse("""<!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN" "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd">
    <parent prop="" id="top"><child name="paul">Text <b>goes</b> here</child>
    <child name="fred">More text</child>
    </parent>""")
    print


